package Webon;

#
# Webon - Software de generación offline de catálogos Web
#
# Copyright (C) 2004/2007 Angel Ortega <angel@venexma.es>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

use locale;

use Template;

# Mensaje de error
our $ERROR = '';

# Versión
our $VERSION = "2.1.1-dev";

# referencia temporal al objeto en vías
# de creación, para usarlo desde los pluguins
our $plugin = undef;

##################################################

sub new
# Crea un objeto Webon
{
	my ($class, %args) = @_;
	my ($w);

	$w = \%args;
	bless($w, $class);

	# añade las opciones de configuración no definidas aún
	$w->{'wrk'}		||= '.';
	$w->{'cat_file'}	||= $w->{'wrk'} . '/catalog';
	$w->{'pref_lang'}	||= 'es';
	$w->{'output'}		||= $w->{'wrk'} . '/output/';
	$w->{'debug'}		||= 0;
	$w->{'verbose'}		||= 0;
	$w->{'simulate'}	||= 0;

	# Crea el objeto Template Toolkit
	unless ($w->{'tt'} = Template->new( {
		'INCLUDE_PATH'	=> join(':',
					("$w->{wrk}/templates",
					 "$w->{wrk}/arts",
					 "$w->{wrk}/pages")
					),
		'INTERPOLATE'	=> 1,
		'TRIM'		=> 1
		})
	)
	{
		$Webon::ERROR = "[Template Toolkit] $Template::ERROR";
		return undef;
	}

	# comprobación de subdirectorios
	foreach my $d ( 'arts', 'img', 'lang', 'pages', 'raw', 'templates', 'th') {
		unless (-d $w->{'wrk'} . "/" . $d) {
			$Webon::ERROR = "$w->{'wrk'}/$d no es un directorio.";
			return undef;
		}
	}

	# idiomas: si no está definida la cadena, se leen del directorio lang
	unless ($w->{'langs'}) {
		my @l = $w->bn_glob("/lang/*");

		# si está el preferido, se quita para añadirlo al principio
		@l = grep { ! /^$w->{'pref_lang'}$/ } @l;
		unshift(@l, $w->{'pref_lang'});

		$w->{'langs'} = join(' ', @l);
	}

	# separa los idiomas
	my @l = split(/ /, $w->{'langs'});
	$w->{'lang'} = \@l;

	# categorías alternativas: la clave del hash contiene ramas
	# del catálogo cuyos valores son listas de nombres de artículos
	# de otras ramas
	# FIXME: para que sea útil, los artículos también han de
	# saber que forman parte de más de una categoría, para añadir
	# sus correspondientes 'migas de pan'.
	$w->{'alt_catalog'} = {};
	#$alt_catalog{cepi} = [ "var_02" ];

	# 'Redirecciones'
	# Cuando una rama del árbol tiene un sólo elemento y ninguna
	# subrama, la página índice se 'redirecciona' con el elemento
	# (en realidad, se copia el fichero del elemento sobre el índice)
	$w->{'redirs'} = {};

	# Carga el catálogo
	unless (open F, $w->{'cat_file'}) {
		$Webon::ERROR = "Imposible abrir catálogo '$w->{'cat_file'}'";
		return undef;
	}

	foreach my $c (<F>) {
		chomp($c);

		# si está en el formato rama1@rama2, rama1
		# será una redirección de rama2
		if (my ($f, $t) = ($c =~ /^(.+)@(.+)$/)) {
			$w->{'redirs'}->{$f} = $t;
			$c = $f;
		}

		# se almacena
		push(@{$w->{'catalog'}}, $c);
	}

	close F;

	# artículos
	my @a = $w->bn_glob("/arts/*");
	$w->{'art_pool'} = \@a;

	# páginas genéricas
	my @p = $w->bn_glob("/pages/*");
	$w->{'pages'} = \@p;

	# imágenes
	my @i = $w->bn_glob("/img/*.jpg");
	$w->{'img_pool'} = \@i;

	# thumbnails
	my @t = $w->bn_glob("/th/*.jpg");
	$w->{'th_pool'} = \@t;

	# documentos en crudo
	my @r = ($w->bn_glob("/raw/*"), $w->bn_glob("/raw/.*"));
	$w->{'raw'} = \@r;

	# registro de uso de imágenes
	$w->{'img_log'} = {};

	# hash de cadenas traducibles
	$w->{'L'} = {};

	# cuenta total de plantillas generadas
	$w->{'stats'} = {
		'new'	=>	0,
		'mod'	=>	0
	};

	# hash de referencias externas (códigos) de artículos
	$w->{art_refs} = {};

	if (open(F, $w->{wrk} . '/refs')) {

		while (<F>) {
			chomp;

			my @l = split(':', $_);

			if ($l[0]) {
				push(@{$w->{art_refs}->{$l[1]}}, [ $l[0], $l[2] ]);
			}
		}

		close F;
	}

	# changelog (fechas de modificación de los artículos)
	$w->{changelog} = [];

	if (open(F, $w->{wrk} . '/changelog')) {

		my @l = reverse sort(<F>);
		close F;

		chomp(@l);
		$w->{changelog} = [ @l ];
	}

	# flujo de funciones a ejecutar al principio
	$w->{'flow'}->{'pre'} = [
		'Webon::setup_output',
		'Webon::transfer_raw'
	];

	# flujo de funciones del bucle principal,
	# dependientes del idioma
	$w->{'flow'}->{'main'} = [
		'Webon::build_superindex',
		'Webon::build_indexes',
		'Webon::build_articles',
		'Webon::build_pages',
		'Webon::build_redirections',
		'Webon::build_news',
		'Webon::build_alpha'
	];

	# flujo de funciones a ejecutar al final
	$w->{'flow'}->{'post'} = [
		'Webon::transfer_images'
	];

	# callbacks para Template Toolkit
	$w->{'tt_callbacks'} = {
		get_sub_branches	=> sub { $w->get_sub_branches(@_); },
		get_articles		=> sub { $w->get_articles(@_); },
		get_images		=> sub { $w->get_images(@_); },
		get_logo		=> sub { $w->get_logo(@_); },
		build_hpath		=> sub { $w->tt_build_hpath(@_); },
		build_hpath_nl		=> sub { $w->tt_build_hpath_nl(@_); },
		build_full_tree		=> sub { $w->tt_build_full_tree(@_); },
		build_full_tree_yahoo	=> sub { $w->tt_build_full_tree_yahoo(@_); },
		img			=> sub { $w->tt_img(@_); },
		img_link		=> sub { $w->tt_imglink(@_); },
		get_article_refs	=> sub { $w->get_article_refs(@_); },
		new_articles		=> sub { $w->tt_new_articles(@_); },
		alpha_index		=> sub { $w->tt_alpha_index(@_); },
		version			=> sub { "<!-- Webon $VERSION -->"; }
	};

	# existe un directorio de plugins?
	if (-d $w->{'wrk'} . "/plugins") {
		# asigna a $Webon::plugin el valor del objeto en curso
		$Webon::plugin = $w;

		# carga y ejecuta los plugins, que probablemente
		# harán algo a $Webon::plugin como añadir
		# funciones a ->{flow}->{main} o similar
		foreach my $p (glob $w->{'wrk'} . "/plugins/*") {
			print "plugin: $p\n" if $w->{'debug'};
			require $p;
		}

		# des-referencia el valor en curso
		$Webon::plugin = undef;
	}

	return $w;
}


sub process_flow
# ejecuta un flujo completo de funciones
{
	my ($w, $flow, $lang) = @_;

	$w->set_lang($lang) if $lang;

	foreach my $func (@{$w->{'flow'}->{$flow}}) {
		print "$flow: $func" . ($lang ? " ($lang)" : "") . "\n"
			if $w->{'debug'};

		&$func($w, $lang);
	}
}


sub process
# efectúa un proceso completo
{
	my ($w) = @_;

	# funciones 'pre'
	$w->process_flow('pre');

	# bucle de idiomas
	foreach my $l (@{$w->{'lang'}}) {
		# funciones 'main'
		$w->process_flow('main', $l);
	}

	# funciones 'post'
	$w->process_flow('post');
}


# funciones 'pre'

sub setup_output
# crea la infraestructura necesaria en el directorio de salida
{
	my ($w) = @_;

	mkdir $w->{'output'};
	mkdir $w->{'output'} . "/img";
	mkdir $w->{'output'} . "/th";
}


sub transfer_raw
# transfiere el contenido del directorio raw
{
	my ($w) = @_;

	foreach my $f (@{$w->{'raw'}}) {
		my ($org) = $w->{'wrk'} . "/raw/" . $f;
		my ($dest) = $w->{'output'} . "/" . $f;

		$w->hardlink_if_diff($org, $dest);
	}
}


# funciones de construcción ('main')

sub build_superindex
# construye el superíndice (raíz)
{
	my ($w, $lang) = @_;

	print "\tSuperindex...\n" if $w->{'verbose'};

	my $vars = $w->default_tt_vars($lang, "index");
	$vars->{'title'} = '';

	$w->process_template('art_INDEX', $vars,
		$w->{'output'} . "/${lang}-index.html");
	$w->process_template('art_INDEX', $vars,
		$w->{'output'} . "/index.html.${lang}");

	# si éste es el idioma preferido, se vuelve a generar
	# la misma página pero como index.html
#	$w->process_template('art_INDEX', $vars,
#		$w->{'output'} . "/index.html")	if $lang eq $w->{'pref_lang'};
}


sub build_indexes
# construye las páginas índice (ramas)
{
	my ($w, $lang) = @_;

	print "\tIndexes...\n" if $w->{'verbose'};

	# recorre el catálogo generando los índices
	foreach my $c (@{$w->{'catalog'}}) {
		my (@a, @b, $logo, $redir);

		# obtiene las subramas de esta rama
		@b = $w->get_sub_branches($c);

		# obtiene los artículos de esta rama
		@a = $w->get_articles($c);

		# si este índice ya está redireccionado, fuera
		if ($w->{redirs}->{$c}) {
			next;
		}

		# si no hay subramas y sólo hay un artículo,
		# se almacena en el hash de 'redirecciones'
		# y no se genera
		if (scalar(@b) == 0 and scalar(@a) == 1) {
			$w->{'redirs'}->{$c} = $a[0];
			next;
		}

		# por la misma razón, si sólo hay una subrama
		# y ningún artículo, también se almacena como
		# redirección
		if (scalar(@b) == 1 and scalar(@a) == 0) {
			$w->{'redirs'}->{$c} = $b[0];
			next;
		}

		my $vars = $w->default_tt_vars($lang, $c);

		$w->process_template('art_index', $vars,
			$w->{'output'} . "/${lang}-${c}.html");
	}
}


sub build_articles
# construye las páginas de artículos (hojas)
{
	my ($w, $lang) = @_;

	print "\tArticles...\n" if $w->{'verbose'};

	# recorre el catálogo
	foreach my $c (@{$w->{'catalog'}}) {
		my @arts = $w->get_articles($c);

		# recorre los artículos de esta rama
		for (my $n = 0; $n < scalar(@arts); $n ++) {
			my $a = $arts[$n];

			# referencias a los artículos anterior y siguiente
			my $prev = $n > 0 ? $arts[$n - 1] : undef;
			my $next = $n < scalar(@arts) + 1 ? $arts[$n + 1] : undef;

			# obtiene la rama a la que pertenece
			my ($path, $code) = ($a =~ /^(.+)_(.+)$/);

			my $vars = $w->default_tt_vars($lang, $a);

			$vars->{'title'} = $w->tt_build_hpath_nl($lang, $path) .
				" : " . $w->{'L'}->{$a};

			$vars->{'art'} = {
				'name' => $w->{'L'}->{$a},
				'path' => $path,
				'prev' => $prev,
				'next' => $next
			};

			$w->process_template('art', $vars,
				$w->{'output'} . "/${lang}-${a}.html");
		}
	}
}


sub build_pages
# construye las páginas genéricas
{
	my ($w, $lang) = @_;

	print "\tPages...\n" if $w->{'verbose'};

	# recorre las páginas
	foreach my $p (@{$w->{'pages'}}) {
		my $o = $p;

		# FIXME: demasiado dependiente de venexma.es
		$o =~ s/_COLOR$//;

		my $title = $w->{'L'}->{$p} || $w->{'L'}->{$o};

		my $vars = $w->default_tt_vars($lang, $p);
		$vars->{'title'} = $title;

		$w->process_template('page', $vars,
			$w->{'output'} . "/${lang}-$p.html");
	}
}


sub build_news
# contruye las páginas de novedades
{
	my ($w, $lang) = @_;

	print "\tNews...\n" if $w->{'verbose'};

	my $vars = $w->default_tt_vars($lang, 'NEW');

	$w->process_template('new', $vars, $w->{'output'} . "/${lang}-NEW.html");
	$w->process_template('rss', $vars, $w->{'output'} . "/rss-${lang}.xml");
}


sub build_alpha
# construye los índices alfabéticos
{
	my ($w, $lang) = @_;

	print "\tAlphabetic indexes...\n" if $w->{'verbose'};

	my $vars = $w->default_tt_vars($lang, 'ABCINDEX');

	$w->process_template('abc', $vars, $w->{'output'} . "/${lang}-ABCINDEX.html");
}


sub build_redirections
# construye las redirecciones
{
	my ($w, $lang) = @_;

	print "\tRedirs...\n" if $w->{'verbose'};

	foreach my $i (keys(%{$w->{'redirs'}})) {
		my ($ifn, $ofn, $c);

		$ifn = $w->{'output'} . '/' . $lang . '-' . $w->{'redirs'}->{$i} . ".html";
		$ofn = $w->{'output'} . '/' . $lang . '-'. $i . ".html";

		if (open F, $ifn) {
			$c = join('', <F>);
			close F;
		}

		# sobreescribe si es diferente
		$w->write_if_diff($ofn, $c);
	}
}


# funciones 'post'

sub transfer_images
# transfiere las imágenes utilizadas
{
	my ($w) = @_;

	foreach my $i (keys(%{$w->{'img_log'}})) {
		my ($org) = $w->{'wrk'} . "/" . $i;
		my ($dest) = $w->{'output'} . "/" . $i;

		$w->hardlink_if_diff($org, $dest);
	}
}

#########################################################

# funciones de ayuda

sub bn_glob
# hace un glob y un basename a la vez de un subdirectorio
# del directorio de trabajo
{
	my ($w, $g) = @_;

	return map { m|/([^/]+)$| } glob $w->{'wrk'} . $g;
}

sub hardlink_if_diff
# crea un enlace simbólico de org a dest si dest
# no existe o es más antiguo que org
{
	my ($w, $org, $dest) = @_;

	# si el origen es un directorio, fuera
	return if -d $org;

	# si no existe el destino o es más antiguo...
	if (! -f $dest or (-M $dest != -M $org)) {
		unless ($w->{'simulate'}) {
			# borra y crea un hardlink
			unlink($dest);
			link($org, $dest);
		}

		print "link: $org => $dest\n" if $w->{'debug'};
	}
}


sub write_if_diff
# sobreescribe el contenido de file si es diferente de lo que hay ahí
{
	my ($w, $file, $content) = @_;
	my ($p);

	# Intenta abrir el ficheroe. Si existe, se
	# compara con el nuevo contenido
	if (open F, $file) {
		$p = join('', <F>);
		close F;

		# si es exactamente igual, fuera
		return 0 if $p eq $content;

		# contabilizar como modificada
		$w->{'stats'}->{'mod'} ++;
	}
	else {
		# una nueva página
		$w->{'stats'}->{'new'} ++;
	}

	# guarda sobre la plantilla
	unless ($w->{'simulate'}) {
		unlink $file;
		open F, ">$file";
		print F $content;
		close F;
	}

	print "\t\t$file\n" if $w->{'verbose'};

	return 1;
}


sub process_template
# Genera una plantilla. Si ha cambiado respecto a la última vez,
# la escribe encima.
{
	my ($w, $t, $v, $o, $c) = @_;
	my ($n, $p);

	# la genera
	$w->{'tt'}->process($t, $v, \$n) or die $Template::ERROR;

	# compacta
	$n =~ s/\t/\n/gm;
	$n =~ s/\n{2,}/\n/gm;
	$n =~ s/ {2,}/ /gm;

	# si $c es un objeto con el método 'convert', se usa
	if (ref($c) and $c->can('convert')) {
		$n = $c->convert($n);
	}

	# escribe (si es necesario)
	$w->write_if_diff($o, $n);
}


sub read_lang_file
# lee un fichero de idiomas
{
	my ($w, $l) = @_;

	unless (open F, $w->{'wrk'} . "/lang/$l") {
		$Template::ERROR = "Imposible abrir fichero de idiomas $l";
		return 0;
	}

	while (<F>) {
		my ($k, $v);

		chomp;

		next if /^#/;

		if (($k, $v) = (/^([^:]+)\s*:\s*(.*)$/) ) {
			$v ||= "{$k}";
			$w->{'L'}->{$k} = $v;
		}
	}

	close F;
	return 1;
}


sub set_lang
# activa un idioma
{
	my ($w, $l) = @_;

	print "Lang: $l\n" if $w->{'verbose'};

	# resetea
	$w->{'L'} = {};

	# carga el idioma preferido
	$w->read_lang_file($w->{'pref_lang'});

	# lo sobreescribe con el solicitado
	$w->read_lang_file($l) if $l ne $w->{'pref_lang'};

	# y lo guarda
	$w->{cur_lang} = $l;
}


sub get_top_level
# devuelve el nivel superior (todas las ramas madre)
{
	my ($w) = @_;

	return grep(/^[^-]+$/, @{$w->{'catalog'}});
}


sub get_sub_branches
# devuelve las subramas de una rama
{
	my ($w, $branch) = @_;
	my (@b);

	# obtiene las subramas de esta rama
	foreach my $b (@{$w->{'catalog'}}) {
		push(@b, $b) if $b =~ /^${branch}\-[^-]+$/;
	}

	return @b;
}


sub get_articles
# devuelve los artículos de una subrama
{
	my ($w, $branch) = @_;
	my (@a);

	# obtiene los artículos de esta rama
	foreach my $a (@{$w->{'art_pool'}}) {
		push(@a, $a) if $a =~ /^${branch}\_/;
	}

	# añade desde las categorías alternativas
	# los posibles artículos de otras ramas
	foreach my $a (@{$w->{'alt_catalog'}->{$branch}}) {
		push(@a, $a);
	}

	return @a;
}


sub get_images
# devuelve las imágenes de un artículo
{
	my ($w, $art) = @_;

	return grep { /^$art-\d+.jpg$/ } @{$w->{'img_pool'}};
}


sub get_logo
# si existe, devuelve una imagen asociada a una rama
{
	my ($w, $b) = @_;

	return grep { /^LOGO-$b.jpg/ } @{$w->{'img_pool'}};
}


sub get_article_refs {
	my ($w, $art) = @_;

	return $w->{art_refs}->{$art};
}

sub default_tt_vars
# devuelve el hash de variables por defecto para Template Toolkit
{
	my ($w, $lang, $this) = @_;

	my $vars = {
		webonid => '',
		lang => $lang,
		langs => $w->{'lang'},
		L => $w->{'L'},
		this => $this,
		title => $w->{'L'}->{$this},
		webon => $w->{'tt_callbacks'}
		};

	return $vars;
}


############################################################

# funciones de ayuda para Template Toolkit

sub tt_build_hpath
# Construye una cadena HTML con la trayectoria 'horizontal'.
{
	my ($w, $lang, $path) = @_;
	my (@out, @l);

	push(@out, "<a href='${lang}-index.html'>$w->{'L'}->{'productos'}</a>");

	foreach my $c (split('-', $path)) {
		my ($l);

		push(@l, $c);
		$l = join("-", @l);
		push(@out, "<a href='${lang}-$l.html'>$w->{'L'}->{$l}</a>");
	}

	return join(" &gt; ", @out);
}


sub tt_build_hpath_nl
# Construye una cadena HTML con la trayectoria 'horizontal' sin enlaces.
{
	my ($w, $lang, $path) = @_;
	my $p = $w->{hpnlc}->{$lang}->{$path};

	if (!$p) {
		my (@out, @l);

		my ($bpath) = ($path =~ /^(.+)_/);

		if (!$bpath) {
			$bpath = $path;
		}

		foreach my $c (split('-', $bpath)) {
			my ($l);

			push(@l, $c);
			$l = join("-", @l);
			push(@out, $w->{'L'}->{$l});
		}

		if ($bpath ne $path) {
			push(@out, $w->{L}->{$path});
		}

		$p = join(" : ", @out);
		$w->{hpnlc}->{$lang}->{$path} = $p;
	}

	return $p;
}


sub tt_build_full_tree
# construye un árbol completo de profundidad variable
{
	my ($w, $lang, $max) = @_;
	my (@out);
	my ($l) = -1;

	foreach my $c (sort { $w->tt_build_hpath_nl($lang, $a) cmp
			$w->tt_build_hpath_nl($lang, $b) } @{$w->{'catalog'}}) {
		# separa en componentes
		my (@comp) = split("-", $c);

		next if $max and scalar(@comp) > $max;

		if ($l < scalar(@comp)) {
			push(@out, "<ul>");
			$l = scalar(@comp);
		}
		elsif ($l > scalar(@comp)) {
			push(@out, "</ul>" x ($l - scalar(@comp)));
			$l = scalar(@comp);
		}

		my ($comp) = $comp[scalar(@comp) - 1];
		push(@out, sprintf("<li><a href='$lang-$c.html'>%s</a></li>",
			$w->{'L'}->{$c}));
	}

	return join("\n", @out);
}


sub tt_build_full_tree_yahoo
# Crea un menú principal tipo Yahoo!
{
	my ($w, $lang, $max) = @_;
	my (@out);
	my ($l) = -1;
	my ($n, $row);

	$n = 0;
	$row = 2;

	push(@out, "<table width='100%'>");

	foreach my $c (sort { $w->tt_build_hpath_nl($lang, $a) cmp
			$w->tt_build_hpath_nl($lang, $b) } $w->get_top_level()) {
		push(@out, "<tr valign=top>") if $n % $row == 0;

		push(@out, "<td>");

		push(@out, sprintf("<a href='$lang-$c.html'>%s</a>",
			$w->{'L'}->{$c}));

		if ($max > 1) {
			my (@t);

			push(@out, "<br><span style='font-size: 9pt'>&nbsp;&nbsp;");

			foreach my $b ($w->get_sub_branches($c)) {
				push(@t, sprintf("<a href='$lang-$b.html'>%s</a>",
					$w->{'L'}->{$b}));
			}

			if (scalar(@t) > 2) {
				@t = @t[0..2];
				push(@out, join(', ', @t) . "...");
			}
			else {
				push(@out, join(', ', @t));
			}

			push(@out, "</span>");
		}

		push(@out, "</td>");

		push(@out, "</tr>") if $n % $row == $row - 1;

		$n++;
	}

	push(@out, "</table>");

	return join("\n", @out);
}


sub tt_img
# crea un enlace <img>, pero además contabilizando la imagen
{
	my ($w, $src, $more) = @_;

	# contabiliza la imagen
	$w->{'img_log'}->{$src} ++;

	return "<img src='$src' " . $more . " >";
}


sub tt_imglink
# crea un enlace a una imagen contabilidándola
{
	my ($w, $src, $dest) = @_;

	# contabiliza la imagen
	$w->{'img_log'}->{$src} ++;

	return "<a href='$src'>" . $dest . "</a>";
}


sub tt_new_articles
# devuelve la lista de nuevos artículos
{
	my ($w, $num) = @_;

	my @l = map {
		my ($mt, $a) = split(':', $_);

		my @p = localtime($mt);
		my $d = sprintf("%d/%d/%d", $p[3], $p[4] + 1, $p[5] + 1900);

		my @ms = qw( Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec );
		my @ws = qw( Sun Mon Tue Wed Thu Fri Sat );

		# Tue, 25 Sep 2007 11:24:36 +0200
		my $rd = sprintf("%s, %d %s %d %02d:%02d:%02d",
			$ws[$p[6]], $p[3], $ms[$p[4]],
			$p[5] + 1900, $p[2], $p[1], $p[0]
		);

		[ $mt, $a, $d, $rd ];

	} @{$w->{changelog}}[0 .. $num];
	return @l;
}


sub tt_alpha_index
{
	my ($w) = @_;
	my %h = ();
	my $ret = '<a name="ABC"></a><p>';

	# construye el índice
	foreach my $a (@{$w->{art_pool}}) {
		# obtiene el nombre traducido del artículo
		my $n = $w->{L}->{$a};

		# obtiene la sección y su traducción
		my ($s) = ($a =~ /^(.+)_/);
		$s = $w->tt_build_hpath_nl($w->{cur_lang}, $s);

		my ($l) = ($n =~ /^(\w)/);

		push(@{$h{uc($l)}}, [ $n, $a, $s ]);
	}

	# letras pulsables (o no)
	foreach my $l (A..Z) {
		if ($h{$l}) {
			$ret .= "<a href='#$l'>$l</a> ";
		}
		else {
			$ret .= $l . ' ';
		}
	}

	$ret .= "</p>\n";

	# índice
	foreach my $l (sort(keys(%h))) {
		$ret .= "<a name='$l'><h2>$l</h2></a>\n<ul>";

		foreach my $a (sort { $a->[0] cmp $b->[0] } @{$h{$l}}) {
			$ret .= "<li><a href='$w->{cur_lang}-$a->[1].html'>";
			$ret .= "$a->[0]</a> ($a->[2])</li>\n";
		}

		$ret .= "</ul>\n";

		$ret .= "<p><a href='#ABC'>^</a></p>";
	}

	return $ret;
}

###################################################

# Webon.pm también puede ejecutarse tal cual

sub as_main
{
	my ($w);

	use Getopt::Long;

	GetOptions("o|output=s" => \$output,
           "c|catalog=s" => \$catalog,
	   "l|langs=s" => \$langs,
	   "d|debug" => \$debug,
	   "w|wrk=s" => \$wrk,
           "n|dry-run" => \$simulate);

	if ($w = new Webon('wrk' => $wrk,
		'debug' => $debug,
		'verbose' => 1,
		'output' => $output,
		'cat_file' => $catalog,
		'simulate' => $simulate)) {
		print "Webon.pm's main\n" if $w->{'debug'};
		$w->process();
	}
	else {
		print $Webon::ERROR . "\n";
	}
}


unless ($INC{'Webon.pm'}) {
	package main;

	Webon::as_main();
}

1;
